/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"
#include "nsIContactProperties.idl"
#include "nsIDOMEventTarget.idl"

interface nsIArray;
interface nsIDOMDOMRequest;
interface nsIDOMDOMCursor;

[scriptable, uuid(72a5ee28-81d8-4af8-90b3-ae935396cc66)]
interface nsIDOMContact : nsIContactProperties
{
  attribute DOMString id;
  readonly attribute jsval     published;
  readonly attribute jsval     updated;

  void init(in nsIContactProperties properties);  // Workaround BUG 723206
};

[scriptable, uuid(e01ebfe7-e972-4e01-b04b-1d162dc74983)]
interface nsIDOMContactManager : nsISupports
{
  nsIDOMDOMRequest find(in nsIContactFindOptions options);

  nsIDOMDOMCursor getAll(in nsIContactFindSortOptions options);

  nsIDOMDOMRequest clear();

  nsIDOMDOMRequest save(in nsIDOMContact contact);

  nsIDOMDOMRequest remove(in nsIDOMContact contact);

  attribute nsIDOMEventListener oncontactchange;

  nsIDOMDOMRequest getRevision();
};
