/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEvent.idl"

/**
 * The nsIDOMCloseEvent interface is the interface to the event
 * close on a WebSocket object.
 *
 * For more information on this interface, please see
 * http://dev.w3.org/html5/websockets/#closeevent
 */
[scriptable, builtinclass, uuid(0b85dc61-2436-4786-b153-097f5c3a33b6)]
interface nsIDOMCloseEvent : nsIDOMEvent
{
  readonly attribute boolean wasClean;
  readonly attribute unsigned short code;
  readonly attribute DOMString reason;

  void initCloseEvent(in DOMString aType,
                      in boolean aCanBubble,
                      in boolean aCancelable,
                      in boolean aWasClean,
                      in unsigned short aReasonCode,
                      in DOMString aReason);
};

dictionary CloseEventInit : EventInit
{
  boolean wasClean;
  unsigned short code;
  DOMString reason;
};
