/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef _mozilla_threads_nsThreadIDs_h_
#define _mozilla_threads_nsThreadIDs_h_

namespace mozilla {
namespace threads {

enum ID {
  Generic = 0,
  Main = 1,
  CycleCollector = 2
};

} /* namespace threads */
} /* namespace mozilla */

#endif /* _mozilla_threads_nsThreadIDs_h_ */
