/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(16ed7adc-b3e0-43c6-a121-5a2b500e90d0)]
interface nsINavigatorBattery : nsISupports
{
  readonly attribute nsISupports battery;
};
