/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

interface nsIDOMTreeWalker;
interface nsIDOMNodeFilter;


[scriptable, uuid(c4ffa159-237c-4dde-b0ba-20b9f9270cf6)]
// Introduced in DOM Level 2:
interface nsIDOMTreeWalker : nsISupports {
  readonly attribute nsIDOMNode       root;
  readonly attribute unsigned long    whatToShow;
  readonly attribute nsIDOMNodeFilter filter;
           attribute nsIDOMNode       currentNode;
                                        // raises(DOMException) on setting

  nsIDOMNode         parentNode();
  nsIDOMNode         firstChild();
  nsIDOMNode         lastChild();
  nsIDOMNode         previousSibling();
  nsIDOMNode         nextSibling();
  nsIDOMNode         previousNode();
  nsIDOMNode         nextNode();
};
