/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "domstubs.idl"

[scriptable, uuid(36f18f12-61a9-4529-8fa9-30050bd6ac00)]
interface nsIDOMSVGAnimatedString
  : nsISupports
{
  attribute DOMString baseVal;
              // raises DOMException on setting
  readonly attribute DOMString animVal;
};
