/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsIDOMEventTarget.idl"

interface nsIDOMDOMCursor;
interface nsIDOMDOMRequest;
interface nsIDOMEventListener;
interface nsIDOMMozSmsFilter;
interface nsIDOMMozSmsSegmentInfo;

[scriptable, builtinclass, uuid(8ce00d77-71b4-43f6-92a1-2eae7c9581b9)]
interface nsIDOMMozSmsManager : nsIDOMEventTarget
{
  nsIDOMMozSmsSegmentInfo getSegmentInfoForText(in DOMString text);

  // The first parameter can be either a DOMString (only one number) or an array
  // of DOMStrings.
  // The method returns a DOMRequest object if one number has been passed.
  // An array of DOMRequest objects otherwise.
  jsval send(in jsval number, in DOMString message);

  [binaryname(GetMessageMoz)]
  nsIDOMDOMRequest getMessage(in long id);

  // The parameter can be either a message id or a SmsMessage.
  nsIDOMDOMRequest delete(in jsval param);

  // Iterates through nsIDOMMozSmsMessage.
  nsIDOMDOMCursor getMessages(in nsIDOMMozSmsFilter filter, in boolean reverse);

  nsIDOMDOMRequest markMessageRead(in long id, in boolean aValue);

  // Iterates through nsIDOMMozMobileMessageThread.
  nsIDOMDOMCursor getThreads();

  [implicit_jscontext] attribute jsval onreceived;
  [implicit_jscontext] attribute jsval onsending;
  [implicit_jscontext] attribute jsval onsent;
  [implicit_jscontext] attribute jsval onfailed;
  [implicit_jscontext] attribute jsval ondeliverysuccess;
  [implicit_jscontext] attribute jsval ondeliveryerror;
};
