/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */


#include "nsISupports.idl"

[scriptable, uuid(20d8eb40-c494-497f-b2a6-aaa32f807ebd)]

/**
 * An object containing metainfo for a login stored by the login manager.
 *
 * Code using login manager can generally ignore this interface. When adding
 * logins, default value will be created. When modifying logins, these
 * properties will be unchanged unless a change is explicitly requested [by
 * using modifyLogin() with a nsIPropertyBag]. When deleting a login or
 * comparing logins, these properties are ignored.
 */
interface nsILoginMetaInfo : nsISupports {
    /**
     * The GUID to uniquely identify the login. This can be any arbitrary
     * string, but a format as created by nsIUUIDGenerator is recommended.
     * For example, "{d4e1a1f6-5ea0-40ee-bff5-da57982f21cf}"
     *
     * addLogin will generate a random value unless a value is provided.
     *
     * addLogin and modifyLogin will throw if the GUID already exists.
     */
    attribute AString guid;

    /**
     * The time, in Unix Epoch milliseconds, when the login was first created.
     */
    attribute unsigned long long timeCreated;

    /**
     * The time, in Unix Epoch milliseconds, when the login was last submitted
     * in a form or used to begin an HTTP auth session.
     */
    attribute unsigned long long timeLastUsed;

    /**
     * The time, in Unix Epoch milliseconds, when the login's password was
     * last modified.
     */
    attribute unsigned long long timePasswordChanged;

    /**
     * The number of times the login was submitted in a form or used to begin
     * an HTTP auth session.
     */
    attribute unsigned long timesUsed;
};
